@@ -14,6 +14,7 @@
 
 LOAD_BALANCER_ROWS = (
     'admin_state_up',
+    'availability_zone',
     'created_at',
     'description',
     'flavor_id',
@@ -39,6 +40,7 @@ LOAD_BALANCER_COLUMNS = (
     'project_id',
     'vip_address',
     'provisioning_status',
+    'operating_status',
     'provider')
 
 LOAD_BALANCER_STATS_ROWS = (
@@ -73,7 +75,11 @@ LISTENER_ROWS = (
     'updated_at',
     'client_ca_tls_container_ref',
     'client_authentication',
-    'client_crl_container_ref')
+    'client_crl_container_ref',
+    'allowed_cidrs',
+    'tls_ciphers',
+    'tls_versions',
+    'alpn_protocols')
 
 LISTENER_COLUMNS = (
     'id',
@@ -104,7 +110,9 @@ POOL_ROWS = (
     'tls_container_ref',
     'ca_tls_container_ref',
     'crl_container_ref',
-    'tls_enabled')
+    'tls_enabled',
+    'tls_ciphers',
+    'tls_versions')
 
 POOL_COLUMNS = (
     'id',
@@ -232,6 +240,8 @@ QUOTA_ROWS = (
     'pool',
     'health_monitor',
     'member',
+    'l7policy',
+    'l7rule',
 )
 
 QUOTA_COLUMNS = (
@@ -241,6 +251,8 @@ QUOTA_COLUMNS = (
     'pool',
     'health_monitor',
     'member',
+    'l7policy',
+    'l7rule',
 )
 
 AMPHORA_ROWS = (
@@ -275,12 +287,25 @@ AMPHORA_COLUMNS = (
     'ha_ip',
 )
 
+AMPHORA_COLUMNS_LONG = (
+    'id',
+    'loadbalancer_id',
+    'status',
+    'role',
+    'lb_network_ip',
+    'ha_ip',
+    'compute_id',
+    'cached_zone',
+    'image_id',
+)
+
 PROVIDER_COLUMNS = (
     'name',
     'description',
 )
 
-PROVIDER_CAPABILICY_COLUMNS = (
+PROVIDER_CAPABILITY_COLUMNS = (
+    'type',
     'name',
     'description',
 )
@@ -312,3 +337,40 @@ FLAVORPROFILE_COLUMNS = (
     'name',
     'provider_name',
 )
+
+AVAILABILITYZONE_ROWS = (
+    'name',
+    'availability_zone_profile_id',
+    'enabled',
+    'description',
+)
+
+AVAILABILITYZONE_COLUMNS = (
+    'name',
+    'availability_zone_profile_id',
+    'enabled',
+)
+
+AVAILABILITYZONEPROFILE_ROWS = (
+    'id',
+    'name',
+    'provider_name',
+    'availability_zone_data'
+)
+
+AVAILABILITYZONEPROFILE_COLUMNS = (
+    'id',
+    'name',
+    'provider_name',
+)
+
+PROVISIONING_STATUS = 'provisioning_status'
+STATUS = 'status'
+
+# TCP/UDP port min/max
+MIN_PORT_NUMBER = 1
+MAX_PORT_NUMBER = 65535
+
+# Member weight min/max
+MIN_WEIGHT = 0
+MAX_WEIGHT = 256
