@@ -1,4 +1,6 @@
 #   Copyright 2017 GoDaddy
+#   Copyright 2019 Red Hat, Inc. All rights reserved.
+#
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
 #   a copy of the License at
@@ -17,22 +19,25 @@
 
 from cliff import lister
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
+from oslo_utils import uuidutils
 
 from octaviaclient.osc.v2 import constants as const
 from octaviaclient.osc.v2 import utils as v2_utils
 
 HTTP_METHODS = ['GET', 'POST', 'DELETE', 'PUT', 'HEAD', 'OPTIONS', 'PATCH',
                 'CONNECT', 'TRACE']
+HTTP_VERSIONS = [1.0, 1.1]
 TYPE_CHOICES = ['PING', 'HTTP', 'TCP', 'HTTPS', 'TLS-HELLO',
-                'UDP-CONNECT']
+                'UDP-CONNECT', 'SCTP']
 
 
 class CreateHealthMonitor(command.ShowOne):
     """Create a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(CreateHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'pool',
@@ -50,6 +55,12 @@ class CreateHealthMonitor(command.ShowOne):
             required=True,
             help="Set the time in seconds, between sending probes to members."
         )
+        parser.add_argument(
+            '--domain-name',
+            metavar='<domain_name>',
+            help=("Set the domain name, which be injected into the HTTP Host "
+                  "Header to the backend server for HTTP health check.")
+        )
         parser.add_argument(
             '--expected-codes',
             metavar='<codes>',
@@ -64,6 +75,13 @@ class CreateHealthMonitor(command.ShowOne):
             help="Set the HTTP method that the health monitor uses for "
                  "requests."
         )
+        parser.add_argument(
+            '--http-version',
+            metavar='<http_version>',
+            choices=HTTP_VERSIONS,
+            type=float,
+            help="Set the HTTP version."
+        )
         parser.add_argument(
             '--timeout',
             metavar='<timeout>',
@@ -116,6 +134,11 @@ class CreateHealthMonitor(command.ShowOne):
             default=None,
             help="Disable health monitor."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -127,6 +150,20 @@ class CreateHealthMonitor(command.ShowOne):
         data = self.app.client_manager.load_balancer.health_monitor_create(
             json=body)
 
+        if parsed_args.wait:
+            pool = self.app.client_manager.load_balancer.pool_show(
+                data['healthmonitor']['pools'][0]['id'])
+            v2_utils.wait_for_active(
+                status_f=(self.app.client_manager.load_balancer.
+                          load_balancer_show),
+                res_id=pool['loadbalancers'][0]['id']
+            )
+            data = {
+                'healthmonitor': (
+                    self.app.client_manager.load_balancer.health_monitor_show(
+                        data['healthmonitor']['id']))
+            }
+
         formatters = {'pools': v2_utils.format_list}
 
         return (rows,
@@ -139,13 +176,18 @@ class DeleteHealthMonitor(command.Command):
     """Delete a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(DeleteHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
             metavar='<health_monitor>',
             help="Health monitor to delete (name or ID)."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -158,12 +200,19 @@ class DeleteHealthMonitor(command.Command):
         self.app.client_manager.load_balancer.health_monitor_delete(
             health_monitor_id=health_monitor_id)
 
+        if parsed_args.wait:
+            v2_utils.wait_for_delete(
+                status_f=(self.app.client_manager.load_balancer.
+                          health_monitor_show),
+                res_id=health_monitor_id
+            )
+
 
 class ListHealthMonitor(lister.Lister):
     """List health monitors"""
 
     def get_parser(self, prog_name):
-        parser = super(ListHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         return parser
 
@@ -184,7 +233,7 @@ class ShowHealthMonitor(command.ShowOne):
     """Show the details of a single health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(ShowHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -196,14 +245,23 @@ class ShowHealthMonitor(command.ShowOne):
 
     def take_action(self, parsed_args):
         rows = const.MONITOR_ROWS
-        attrs = v2_utils.get_health_monitor_attrs(self.app.client_manager,
-                                                  parsed_args)
+        data = None
+        if uuidutils.is_uuid_like(parsed_args.health_monitor):
+            try:
+                data = (
+                    self.app.client_manager.load_balancer.health_monitor_show(
+                        health_monitor_id=parsed_args.health_monitor))
+            except exceptions.NotFound:
+                pass
+        if data is None:
+            attrs = v2_utils.get_health_monitor_attrs(self.app.client_manager,
+                                                      parsed_args)
 
-        health_monitor_id = attrs.pop('health_monitor_id')
+            health_monitor_id = attrs.pop('health_monitor_id')
 
-        data = self.app.client_manager.load_balancer.health_monitor_show(
-            health_monitor_id=health_monitor_id,
-        )
+            data = self.app.client_manager.load_balancer.health_monitor_show(
+                health_monitor_id=health_monitor_id,
+            )
         formatters = {'pools': v2_utils.format_list}
 
         return (rows,
@@ -214,7 +272,7 @@ class SetHealthMonitor(command.Command):
     """Update a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(SetHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -231,6 +289,12 @@ class SetHealthMonitor(command.Command):
             metavar='<delay>',
             help="Set the time in seconds, between sending probes to members."
         )
+        parser.add_argument(
+            '--domain-name',
+            metavar='<domain_name>',
+            help=("Set the domain name, which be injected into the HTTP Host "
+                  "Header to the backend server for HTTP health check.")
+        )
         parser.add_argument(
             '--expected-codes',
             metavar='<codes>',
@@ -245,6 +309,13 @@ class SetHealthMonitor(command.Command):
             help="Set the HTTP method that the health monitor uses for "
                  "requests."
         )
+        parser.add_argument(
+            '--http-version',
+            metavar='<http_version>',
+            choices=HTTP_VERSIONS,
+            type=float,
+            help="Set the HTTP version."
+        )
         parser.add_argument(
             '--timeout',
             metavar='<timeout>',
@@ -287,6 +358,11 @@ class SetHealthMonitor(command.Command):
             default=None,
             help="Disable health monitor."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -294,9 +370,92 @@ class SetHealthMonitor(command.Command):
         attrs = v2_utils.get_health_monitor_attrs(self.app.client_manager,
                                                   parsed_args)
 
-        listener_id = attrs.pop('health_monitor_id')
+        hm_id = attrs.pop('health_monitor_id')
 
         body = {'healthmonitor': attrs}
 
         self.app.client_manager.load_balancer.health_monitor_set(
-            listener_id, json=body)
+            hm_id, json=body)
+
+        if parsed_args.wait:
+            v2_utils.wait_for_active(
+                status_f=(self.app.client_manager.load_balancer.
+                          health_monitor_show),
+                res_id=hm_id
+            )
+
+
+class UnsetHealthMonitor(command.Command):
+    """Clear health monitor settings"""
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+
+        parser.add_argument(
+            'health_monitor',
+            metavar='<health_monitor>',
+            help="Health monitor to update (name or ID)."
+        )
+        parser.add_argument(
+            '--domain-name',
+            action='store_true',
+            help="Clear the health monitor domain name."
+        )
+        parser.add_argument(
+            '--expected-codes',
+            action='store_true',
+            help="Reset the health monitor expected codes to the API default."
+        )
+        parser.add_argument(
+            '--http-method',
+            action='store_true',
+            help="Reset the health monitor HTTP method to the API default."
+        )
+        parser.add_argument(
+            '--http-version',
+            action='store_true',
+            help="Reset the health monitor HTTP version to the API default."
+        )
+        parser.add_argument(
+            '--max-retries-down',
+            action='store_true',
+            help="Reset the health monitor max retries down to the API "
+                 "default."
+        )
+        parser.add_argument(
+            '--name',
+            action='store_true',
+            help="Clear the health monitor name."
+        )
+        parser.add_argument(
+            '--url-path',
+            action='store_true',
+            help="Clear the health monitor URL path."
+        )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        unset_args = v2_utils.get_unsets(parsed_args)
+        if not unset_args:
+            return
+
+        hm_id = v2_utils.get_resource_id(
+            self.app.client_manager.load_balancer.health_monitor_list,
+            'healthmonitors', parsed_args.health_monitor)
+
+        body = {'healthmonitor': unset_args}
+
+        self.app.client_manager.load_balancer.health_monitor_set(
+            hm_id, json=body)
+
+        if parsed_args.wait:
+            v2_utils.wait_for_active(
+                status_f=(self.app.client_manager.load_balancer.
+                          health_monitor_show),
+                res_id=hm_id
+            )
