@@ -12,9 +12,13 @@
 #   under the License.
 #
 
-from osc_lib import exceptions
-
+import munch
 from openstackclient.identity import common as identity_common
+from osc_lib import exceptions as osc_exc
+from osc_lib import utils
+
+from octaviaclient.api import exceptions
+from octaviaclient.osc.v2 import constants
 
 
 def _map_attrs(args, source_attr_map):
@@ -76,6 +80,11 @@ def get_resource_id(resource, resource_name, name):
             if name.lower() in ('none', 'null', 'void'):
                 return None
 
+        primary_key = 'id'
+        # Availability-zones don't have an id value
+        if resource_name == 'availability_zones':
+            primary_key = 'name'
+
         # Projects can be non-uuid so we need to account for this
         if resource_name == 'project':
             if name != 'non-uuid':
@@ -84,38 +93,35 @@ def get_resource_id(resource, resource_name, name):
                     name
                 ).id
                 return project_id
-            else:
-                return 'non-uuid'
-        elif resource_name == 'members':
+            return 'non-uuid'
+        if resource_name == 'members':
             names = [re for re in resource(name['pool_id'])['members']
-                     if re.get('id') == name['member_id']
-                     or re.get('name') == name['member_id']]
+                     if re.get('id') == name['member_id'] or
+                     re.get('name') == name['member_id']]
             name = name['member_id']
             if len(names) > 1:
                 msg = ("{0} {1} found with name or ID of {2}. Please try "
                        "again with UUID".format(len(names), resource_name,
                                                 name))
-                raise exceptions.CommandError(msg)
-            else:
-                return names[0].get('id')
-        elif resource_name == 'l7rules':
+                raise osc_exc.CommandError(msg)
+            return names[0].get('id')
+        if resource_name == 'l7rules':
             names = [re for re in resource(name['l7policy_id'])['rules']
                      if re.get('id') == name['l7rule_id']]
             name = name['l7rule_id']
             return names[0].get('id')
-        else:
-            names = [re for re in resource()[resource_name]
-                     if re.get('name') == name or re.get('id') == name]
-            if len(names) > 1:
-                msg = ("{0} {1} found with name or ID of {2}. Please try "
-                       "again with UUID".format(len(names), resource_name,
-                                                name))
-                raise exceptions.CommandError(msg)
-            else:
-                return names[0].get('id')
-    except IndexError:
+        names = [re for re in resource()[resource_name]
+                 if re.get('name') == name or re.get('id') == name]
+        if len(names) > 1:
+            msg = ("{0} {1} found with name or ID of {2}. Please try "
+                   "again with UUID".format(len(names), resource_name,
+                                            name))
+            raise osc_exc.CommandError(msg)
+        return names[0].get(primary_key)
+
+    except IndexError as e:
         msg = "Unable to locate {0} in {1}".format(name, resource_name)
-        raise exceptions.CommandError(msg)
+        raise osc_exc.CommandError(msg) from e
 
 
 def get_loadbalancer_attrs(client_manager, parsed_args):
@@ -161,7 +167,14 @@ def get_loadbalancer_attrs(client_manager, parsed_args):
         'cascade': ('cascade', lambda x: True),
         'provisioning_status': ('provisioning_status', str),
         'operating_status': ('operating_status', str),
-        'provider': ('provider', str)
+        'provider': ('provider', str),
+        'flavor': (
+            'flavor_id',
+            'flavors',
+            client_manager.load_balancer.flavor_list
+        ),
+        'availability_zone': ('availability_zone', str),
+
     }
 
     _attrs = vars(parsed_args)
@@ -206,6 +219,15 @@ def get_listener_attrs(client_manager, parsed_args):
         'timeout_member_connect': ('timeout_member_connect', int),
         'timeout_member_data': ('timeout_member_data', int),
         'timeout_tcp_inspect': ('timeout_tcp_inspect', int),
+        'client_ca_tls_container_ref': ('client_ca_tls_container_ref',
+                                        _format_str_if_need_treat_unset),
+        'client_authentication': ('client_authentication', str),
+        'client_crl_container_ref': ('client_crl_container_ref',
+                                     _format_str_if_need_treat_unset),
+        'allowed_cidrs': ('allowed_cidrs', list),
+        'tls_ciphers': ('tls_ciphers', str),
+        'tls_versions': ('tls_versions', list),
+        'alpn_protocols': ('alpn_protocols', list),
     }
 
     _attrs = vars(parsed_args)
@@ -242,7 +264,18 @@ def get_pool_attrs(client_manager, parsed_args):
         ),
         'session_persistence': ('session_persistence', _format_kv),
         'enable': ('admin_state_up', lambda x: True),
-        'disable': ('admin_state_up', lambda x: False)
+        'disable': ('admin_state_up', lambda x: False),
+        'tls_container_ref': ('tls_container_ref',
+                              _format_str_if_need_treat_unset),
+        'ca_tls_container_ref': ('ca_tls_container_ref',
+                                 _format_str_if_need_treat_unset),
+        'crl_container_ref': ('crl_container_ref',
+                              _format_str_if_need_treat_unset),
+
+        'enable_tls': ('tls_enabled', lambda x: True),
+        'disable_tls': ('tls_enabled', lambda x: False),
+        'tls_ciphers': ('tls_ciphers', str),
+        'tls_versions': ('tls_versions', list),
     }
 
     _attrs = vars(parsed_args)
@@ -297,6 +330,8 @@ def get_l7policy_attrs(client_manager, parsed_args):
         'name': ('name', str),
         'description': ('description', str),
         'redirect_url': ('redirect_url', str),
+        'redirect_http_code': ('redirect_http_code', int),
+        'redirect_prefix': ('redirect_prefix', str),
         'l7policy': (
             'l7policy_id',
             'l7policies',
@@ -390,7 +425,9 @@ def get_health_monitor_attrs(client_manager, parsed_args):
         'max_retries_down': ('max_retries_down', int),
         'url_path': ('url_path', str),
         'enable': ('admin_state_up', lambda x: True),
-        'disable': ('admin_state_up', lambda x: False)
+        'disable': ('admin_state_up', lambda x: False),
+        'http_version': ('http_version', float),
+        'domain_name': ('domain_name', str)
     }
 
     _attrs = vars(parsed_args)
@@ -406,6 +443,8 @@ def get_quota_attrs(client_manager, parsed_args):
         'load_balancer': ('load_balancer', int),
         'member': ('member', int),
         'pool': ('pool', int),
+        'l7policy': ('l7policy', int),
+        'l7rule': ('l7rule', int),
         'project': (
             'project_id',
             'project',
@@ -439,15 +478,113 @@ def get_amphora_attrs(client_manager, parsed_args):
     return _map_attrs(vars(parsed_args), attr_map)
 
 
+def get_provider_attrs(parsed_args):
+    attr_map = {
+        'provider': ('provider_name', str),
+        'description': ('description', str),
+        'flavor': ('flavor', bool),
+        'availability_zone': ('availability_zone', bool),
+    }
+
+    return _map_attrs(vars(parsed_args), attr_map)
+
+
+def get_flavor_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'flavor': (
+            'flavor_id',
+            'flavors',
+            client_manager.load_balancer.flavor_list,
+        ),
+        'flavorprofile': (
+            'flavor_profile_id',
+            'flavorprofiles',
+            client_manager.load_balancer.flavorprofile_list,
+        ),
+        'enable': ('enabled', lambda x: True),
+        'disable': ('enabled', lambda x: False),
+        'description': ('description', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
+def get_flavorprofile_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'flavorprofile': (
+            'flavorprofile_id',
+            'flavorprofiles',
+            client_manager.load_balancer.flavorprofile_list,
+        ),
+        'provider': ('provider_name', str),
+        'flavor_data': ('flavor_data', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
+def get_availabilityzone_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'availabilityzone': (
+            'availabilityzone_name',
+            'availability_zones',
+            client_manager.load_balancer.availabilityzone_list,
+        ),
+        'availabilityzoneprofile': (
+            'availability_zone_profile_id',
+            'availability_zone_profiles',
+            client_manager.load_balancer.availabilityzoneprofile_list,
+        ),
+        'enable': ('enabled', lambda x: True),
+        'disable': ('enabled', lambda x: False),
+        'description': ('description', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
+def get_availabilityzoneprofile_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'availabilityzoneprofile': (
+            'availability_zone_profile_id',
+            'availability_zone_profiles',
+            client_manager.load_balancer.availabilityzoneprofile_list,
+        ),
+        'provider': ('provider_name', str),
+        'availability_zone_data': ('availability_zone_data', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
 def format_list(data):
     return '\n'.join(i['id'] for i in data)
 
 
+def format_list_flat(data):
+    return '\n'.join(i for i in data)
+
+
 def format_hash(data):
     if data:
         return '\n'.join('{}={}'.format(k, v) for k, v in data.items())
-    else:
-        return None
+    return None
 
 
 def _format_kv(data):
@@ -458,3 +595,50 @@ def _format_kv(data):
         formatted_kv[k] = v
 
     return formatted_kv
+
+
+def _format_str_if_need_treat_unset(data):
+    if data.lower() in ('none', 'null', 'void'):
+        return None
+    return str(data)
+
+
+def get_unsets(parsed_args):
+    return {arg: None for arg, value in vars(parsed_args).items() if
+            value is True and arg != 'wait'}
+
+
+def wait_for_active(status_f, res_id):
+    success = utils.wait_for_status(
+        status_f=lambda x: munch.Munch(status_f(x)),
+        res_id=res_id,
+        status_field=constants.PROVISIONING_STATUS,
+        sleep_time=3
+    )
+    if not success:
+        raise exceptions.OctaviaClientException(
+            code="n/a",
+            message="The resource did not successfully reach ACTIVE status.")
+
+
+def wait_for_delete(status_f, res_id,
+                    status_field=constants.PROVISIONING_STATUS):
+    class Getter(object):
+        @staticmethod
+        def get(id):
+            return munch.Munch(status_f(id))
+
+    try:
+        success = utils.wait_for_delete(
+            manager=Getter,
+            res_id=res_id,
+            status_field=status_field,
+            sleep_time=3
+        )
+        if not success:
+            raise exceptions.OctaviaClientException(
+                code="n/a",
+                message="The resource could not be successfully deleted.")
+    except exceptions.OctaviaClientException as e:
+        if e.code != 404:
+            raise
